==> ec2init-0.17-py3-none-any.whl: ec2init/plans/multinode/generate_opensearch_users_passwords.py <==
from ec2init import Procedure

from configparser import ConfigParser
from pathlib import Path
from typing import List, Dict
import logging
from ec2init.plans.init.configure_glassfish import ConfigureGlassfish
from ec2init.plans.multinode.master_seds import MasterSeds
from ec2init.utils.database import databaseUtils
import yaml 
from yaml.loader import SafeLoader
from ec2init.utils.services import Service, ServiceUtils
from ec2init.utils.system import SystemUtils
import xml.etree.ElementTree as ET
import os 
from ec2init.plans.init.start_services import StartServices

logger = logging.getLogger(__name__)

class GenerateOpensearchUsersPasswords(Procedure):
    def __init__(self, name: str, config: ConfigParser, check_file: Path) -> None:
        self.name = name
        self.config = config
        self.check_file = check_file
        self.consul_kv_key_logstash_pass = "opensearch/user/logstash/pass"
        self.consul_kv_key_elastic_exporter_pass = "opensearch/user/elastic_exporter/pass"
        self.tmp_password_file = "/tmp/opensearch_users_pass"
        self._write_password_to_file('', self.tmp_password_file)
        os.chmod(self.tmp_password_file, 0o755)

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        self._generate_internal_users_passwords()
        self._run_security_admin()

        if self.check_file is not None:
            self.check_file.touch()
        logger.info("%s - Finished execution", self.name)
        return []

    def _run_security_admin(self) -> None:
        run_securityadmin_script = "/srv/hops/opensearch/plugins/opensearch-security/tools/run_securityAdmin.sh"
        logger.info("%s - runing elastic securityAdmin", self.name)
        SystemUtils.execute_command_with_retry([run_securityadmin_script], shell=True, timeout=None)

        SystemUtils.execute_command(["systemctl", "daemon-reload"])
        for service_to_restart in [Service.Glassfish, Service.Logstash, Service.OpensearchDashboards, Service.ePipe, Service.ElasticExporter]:
            if StartServices._is_service_running(service_to_restart.value):
                timeout = 60
                if service_to_restart == Service.Glassfish:
                    timeout = ConfigureGlassfish.GLASSFISH_START_TIMEOUT
                SystemUtils.execute_command(["systemctl", "restart", service_to_restart.value], timeout=timeout)
        if StartServices.is_service_installed("consul") and StartServices.is_service_enabled("consul"):
            SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _generate_internal_users_passwords(self) -> None:
        internal_users_yml = "/srv/hops/opensearch/plugins/opensearch-security/securityconfig/internal_users.yml"
        with open(internal_users_yml, 'r') as f:
            internal_users_dict : Dict  = yaml.load(f, Loader=SafeLoader)
        
        admin_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["admin"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_admin(admin_password)

        kibanaserver_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["kibanaserver"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_kibanauser(kibanaserver_password)
        
        logstash_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["logstash"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_logstash(self.tmp_password_file)
        self._add_pass_to_consul_kv(self.consul_kv_key_logstash_pass, self.tmp_password_file)

        elastic_exporter_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["elastic_exporter"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_elastic_exporter(self.tmp_password_file)
        self._add_pass_to_consul_kv(self.consul_kv_key_elastic_exporter_pass, self.tmp_password_file)

        epipeuser_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["epipeuser"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_epipeuser(self.tmp_password_file)

        service_log_viewer_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["service_log_viewer"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_expat(admin_password, service_log_viewer_password)

        with open(internal_users_yml, 'w') as f:
            yaml.dump(internal_users_dict, f, sort_keys=False, default_flow_style=False)

        os.remove(self.tmp_password_file)

    def _generate_password(self, password_file :str) -> str:
        process = SystemUtils.execute_command(["tr -dc A-Za-z0-9 </dev/urandom | head -c 30 ; echo ''"], shell=True)
        password = process.stdout.strip().decode("utf-8")
        self._write_password_to_file(password, password_file)
        return password

    def _write_password_to_file(self, password: str, password_file: str) -> None:
        with open(password_file, "w+") as fd:
            fd.write(password)
        
    def _hash_password(self, password_file :str) -> str:
        process = SystemUtils.execute_command([f"/srv/hops/opensearch/plugins/opensearch-security/tools/hash.sh -p `cat {password_file}`"], shell=True)
        return process.stdout.strip().decode("utf-8")

    def _update_admin(self, password : str) -> None:
        databaseUtils.update_variable("elastic_admin_password", password)

    def _update_kibanauser(self, password : str) -> None:
        opensearch_dashboards_config_file =  "/srv/hops/opensearch-dashboards/config/opensearch_dashboards.yml"
        with open(opensearch_dashboards_config_file, 'r') as f:
            opensearch_dashboards_config : Dict  = yaml.load(f, Loader=yaml.loader.SafeLoader)
        opensearch_dashboards_config["opensearch.password"] = password

        with open(opensearch_dashboards_config_file, 'w') as f:
            yaml.dump(opensearch_dashboards_config, f, sort_keys=False, default_flow_style=False)

    def _update_logstash(self, password_file : str) -> None:
        for pipeline in MasterSeds.LOGSTASH_PIPELINES:
            SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i 's/password =>.*/password => \"'\"$PASSWORD\"'\"/g' /srv/hops/logstash/config/{pipeline}.conf"], shell=True)
    
    def _update_elastic_exporter(self, password_file : str) -> None:
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'elastic_exporter:.*@'/\"elastic_exporter:$PASSWORD@\"/g /lib/systemd/system/elastic_exporter.service"], shell=True)
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'elastic_exporter:.*@'/\"elastic_exporter:$PASSWORD@\"/g /srv/hops/consul/consul.d/elastic-consul.hcl"], shell=True)
        SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _update_epipeuser(self, password_file :str) -> None:
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'password =.*'/\"password = $PASSWORD\"/g /srv/hops/epipe/conf/config.ini"], shell=True)

    def _update_expat(self, admin_password :str, service_log_viewer_password :str) -> None:
        expat_site = "/root/expat/etc/expat-site.xml"
        tree = ET.parse(expat_site)
        e = tree.getroot().iter('elastic').__next__()
        e_pass = e.find("pass")
        if e_pass is not None:
            e_pass.text = admin_password
        e_services_logs_pass = e.find("services-logs-pass")
        if e_services_logs_pass is not None:
            e_services_logs_pass.text = service_log_viewer_password
        tree.write(open(expat_site, 'w'), encoding='unicode')

    def _add_pass_to_consul_kv(self, key :str, password_file :str) -> None:
        SystemUtils.execute_command([f"su - consul -c 'source ~/.bashrc && /srv/hops/consul/bin/consul kv put {key} @{password_file} '"], shell=True)
      

    def rollback(self) -> None:
        pass


class UpdateOpensearchUsersPasswords(GenerateOpensearchUsersPasswords):
    def __init__(self, name: str, config: ConfigParser, check_file: Path, services_list: List[Service]) -> None:
        super().__init__(name, config, check_file)
        self.services_list = services_list

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        
        if ServiceUtils.containsKey(self.services_list, Service.Logstash):
            logstash_pass = self._get_pass_from_consul_kv(self.consul_kv_key_logstash_pass)
            self._write_password_to_file(logstash_pass, self.tmp_password_file)
            self._update_logstash(self.tmp_password_file)
            SystemUtils.execute_command(["systemctl", "restart", "logstash"])

        if ServiceUtils.containsKey(self.services_list, Service.ElasticExporter):
            elastic_exporter_pass = self._get_pass_from_consul_kv(self.consul_kv_key_elastic_exporter_pass)
            self._write_password_to_file(elastic_exporter_pass, self.tmp_password_file)
            self._update_elastic_exporter(self.tmp_password_file)
            SystemUtils.execute_command(["systemctl", "daemon-reload"])
            SystemUtils.execute_command(["systemctl", "restart", "elastic_exporter"])

        os.remove(self.tmp_password_file)

        if self.check_file is not None:
            self.check_file.touch()
        logger.info("%s - Finished execution", self.name)
        return []

    def _get_pass_from_consul_kv(self, key :str) -> str:
        process = SystemUtils.execute_command([f"su - consul -c 'source ~/.bashrc && /srv/hops/consul/bin/consul kv get {key} '"], shell=True)
        return process.stdout.decode("utf-8").strip()

==> ec2init-0.19.2-py3-none-any.whl: ec2init/plans/multinode/generate_opensearch_users_passwords.py <==
from ec2init import Procedure

from configparser import ConfigParser
from pathlib import Path
from typing import List, Dict
import logging
from ec2init.plans.init.configure_glassfish import ConfigureGlassfish
from ec2init.plans.multinode.master_seds import MasterSeds
from ec2init.utils.database import databaseUtils
import yaml 
from yaml.loader import SafeLoader
from ec2init.utils.services import Service, ServiceUtils
from ec2init.utils.system import SystemUtils
import xml.etree.ElementTree as ET
import os 
from ec2init.plans.init.start_services import StartServices
from ec2init.utils.systemd import SystemdUtils

logger = logging.getLogger(__name__)

class GenerateOpensearchUsersPasswords(Procedure):
    def __init__(self, name: str, config: ConfigParser, check_file: Path) -> None:
        self.name = name
        self.config = config
        self.check_file = check_file
        self.consul_kv_key_logstash_pass = "opensearch/user/logstash/pass"
        self.consul_kv_key_elastic_exporter_pass = "opensearch/user/elastic_exporter/pass"
        self.tmp_password_file = "/tmp/opensearch_users_pass"
        self._write_password_to_file('', self.tmp_password_file)
        os.chmod(self.tmp_password_file, 0o755)

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        self._generate_internal_users_passwords()
        self._run_security_admin()

        if self.check_file is not None:
            self.check_file.touch()
        logger.info("%s - Finished execution", self.name)
        return []

    def _run_security_admin(self) -> None:
        run_securityadmin_script = "/srv/hops/opensearch/plugins/opensearch-security/tools/run_securityAdmin.sh"
        logger.info("%s - runing elastic securityAdmin", self.name)
        SystemUtils.execute_command_with_retry([run_securityadmin_script], shell=True, timeout=None)

        SystemUtils.execute_command(["systemctl", "daemon-reload"])
        for service_to_restart in [Service.Glassfish, Service.Logstash, Service.OpensearchDashboards, Service.ePipe, Service.ElasticExporter]:
            if SystemdUtils.is_active(service_to_restart.value):
                timeout = 60
                if service_to_restart == Service.Glassfish:
                    timeout = ConfigureGlassfish.GLASSFISH_START_TIMEOUT
                SystemUtils.execute_command(["systemctl", "restart", service_to_restart.value], timeout=timeout)
        if SystemdUtils.is_service_installed("consul") and SystemdUtils.is_enabled("consul"):
            SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _generate_internal_users_passwords(self) -> None:
        internal_users_yml = "/srv/hops/opensearch/plugins/opensearch-security/securityconfig/internal_users.yml"
        with open(internal_users_yml, 'r') as f:
            internal_users_dict : Dict  = yaml.load(f, Loader=SafeLoader)
        
        admin_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["admin"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_admin(admin_password)

        kibanaserver_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["kibanaserver"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_kibanauser(kibanaserver_password)
        
        logstash_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["logstash"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_logstash(self.tmp_password_file)
        self._add_pass_to_consul_kv(self.consul_kv_key_logstash_pass, self.tmp_password_file)

        elastic_exporter_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["elastic_exporter"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_elastic_exporter(self.tmp_password_file)
        self._add_pass_to_consul_kv(self.consul_kv_key_elastic_exporter_pass, self.tmp_password_file)

        epipeuser_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["epipeuser"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_epipeuser(self.tmp_password_file)

        service_log_viewer_password = self._generate_password(self.tmp_password_file)
        internal_users_dict["service_log_viewer"]["hash"] = self._hash_password(self.tmp_password_file)
        self._update_expat(admin_password, service_log_viewer_password)

        with open(internal_users_yml, 'w') as f:
            yaml.dump(internal_users_dict, f, sort_keys=False, default_flow_style=False)

        os.remove(self.tmp_password_file)

    def _generate_password(self, password_file :str) -> str:
        process = SystemUtils.execute_command(["tr -dc A-Za-z0-9 </dev/urandom | head -c 30 ; echo ''"], shell=True)
        password = process.stdout.strip().decode("utf-8")
        self._write_password_to_file(password, password_file)
        return password

    def _write_password_to_file(self, password: str, password_file: str) -> None:
        with open(password_file, "w+") as fd:
            fd.write(password)
        
    def _hash_password(self, password_file :str) -> str:
        process = SystemUtils.execute_command([f"/srv/hops/opensearch/plugins/opensearch-security/tools/hash.sh -p `cat {password_file}`"], shell=True)
        return process.stdout.strip().decode("utf-8")

    def _update_admin(self, password : str) -> None:
        databaseUtils.update_variable("elastic_admin_password", password)

    def _update_kibanauser(self, password : str) -> None:
        opensearch_dashboards_config_file =  "/srv/hops/opensearch-dashboards/config/opensearch_dashboards.yml"
        with open(opensearch_dashboards_config_file, 'r') as f:
            opensearch_dashboards_config : Dict  = yaml.load(f, Loader=yaml.loader.SafeLoader)
        opensearch_dashboards_config["opensearch.password"] = password

        with open(opensearch_dashboards_config_file, 'w') as f:
            yaml.dump(opensearch_dashboards_config, f, sort_keys=False, default_flow_style=False)

    def _update_logstash(self, password_file : str) -> None:
        for pipeline in MasterSeds.LOGSTASH_PIPELINES:
            SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i 's/password =>.*/password => \"'\"$PASSWORD\"'\"/g' /srv/hops/logstash/config/{pipeline}.conf"], shell=True)
    
    def _update_elastic_exporter(self, password_file : str) -> None:
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'elastic_exporter:.*@'/\"elastic_exporter:$PASSWORD@\"/g /lib/systemd/system/elastic_exporter.service"], shell=True)
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'elastic_exporter:.*@'/\"elastic_exporter:$PASSWORD@\"/g /srv/hops/consul/consul.d/elastic-consul.hcl"], shell=True)
        SystemUtils.execute_command(["systemctl", "reload", "consul"])

    def _update_epipeuser(self, password_file :str) -> None:
        SystemUtils.execute_command([f"PASSWORD=$(cat {password_file}) && sed -i s/'password =.*'/\"password = $PASSWORD\"/g /srv/hops/epipe/conf/config.ini"], shell=True)

    def _update_expat(self, admin_password :str, service_log_viewer_password :str) -> None:
        expat_site = "/root/expat/etc/expat-site.xml"
        tree = ET.parse(expat_site)
        e = tree.getroot().iter('elastic').__next__()
        e_pass = e.find("pass")
        if e_pass is not None:
            e_pass.text = admin_password
        e_services_logs_pass = e.find("services-logs-pass")
        if e_services_logs_pass is not None:
            e_services_logs_pass.text = service_log_viewer_password
        tree.write(open(expat_site, 'w'), encoding='unicode')

    def _add_pass_to_consul_kv(self, key :str, password_file :str) -> None:
        SystemUtils.execute_command([f"su - consul -c 'source ~/.bashrc && /srv/hops/consul/bin/consul kv put {key} @{password_file} '"], shell=True)
      

    def rollback(self) -> None:
        pass


class UpdateOpensearchUsersPasswords(GenerateOpensearchUsersPasswords):
    def __init__(self, name: str, config: ConfigParser, check_file: Path, services_list: List[Service]) -> None:
        super().__init__(name, config, check_file)
        self.services_list = services_list

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        
        if ServiceUtils.containsKey(self.services_list, Service.Logstash):
            logstash_pass = self._get_pass_from_consul_kv(self.consul_kv_key_logstash_pass)
            self._write_password_to_file(logstash_pass, self.tmp_password_file)
            self._update_logstash(self.tmp_password_file)
            SystemUtils.execute_command(["systemctl", "restart", "logstash"])

        if ServiceUtils.containsKey(self.services_list, Service.ElasticExporter):
            elastic_exporter_pass = self._get_pass_from_consul_kv(self.consul_kv_key_elastic_exporter_pass)
            self._write_password_to_file(elastic_exporter_pass, self.tmp_password_file)
            self._update_elastic_exporter(self.tmp_password_file)
            SystemUtils.execute_command(["systemctl", "daemon-reload"])
            SystemUtils.execute_command(["systemctl", "restart", "elastic_exporter"])

        os.remove(self.tmp_password_file)

        if self.check_file is not None:
            self.check_file.touch()
        logger.info("%s - Finished execution", self.name)
        return []

    def _get_pass_from_consul_kv(self, key :str) -> str:
        process = SystemUtils.execute_command([f"su - consul -c 'source ~/.bashrc && /srv/hops/consul/bin/consul kv get {key} '"], shell=True)
        return process.stdout.decode("utf-8").strip()
